import pygame

from text import Text
from input import Input
from buttons import Button

pygame.init()

width, height = 1280, 720
fps = 60

win = pygame.display.set_mode((width, height), pygame.SRCALPHA)
pygame.display.set_caption('widgets test')
clock = pygame.time.Clock()

font = pygame.font.Font(None, 32)

bt = Button(win, x = 100, y = 100, width = 200, fontSize = 60, borderRadius = 25, text = 'Click!')
input = Input(win, x = 100, y = 400, borderRadius = 15, maxChars = 256, noText = 'Enter your text...', cursorColor = (255, 0, 0))
tx = Text(win, x = 450, y = 100, cursorColor = (255, 0, 0), fontSize = 50, color = (0, 150, 255, 128))

while 1:
    clock.tick(fps)
    win.fill((60, 60, 60))
    tx.Scroll()

    for event in pygame.event.get():
        if event.type == pygame.QUIT:
            exit()
        elif event.type == pygame.KEYDOWN:
            input.Press(event.unicode)
            tx.Press(event.unicode)
    
    bt.update()
    input.update()
    tx.update()

    fpsINFO = font.render(str(int(clock.get_fps())), 1, (255, 255, 255))
    win.blit(fpsINFO, (10, 10))
    
    pygame.display.update()


import pygame

pygame.init()

class Text:
    def __init__(self, surface, **args):
        
        Texts.append(self)
        
        self.x, self.y = 0, 0   # начальное положение text
        self.width, self.height = 300, 300   # размер text - (ширина, высота)

        self.surface = surface   # окно на котором будет рисоваться кнопка
        self.renderSurface = pygame.Surface((self.width, self.height), pygame.SRCALPHA)   # второе окно к котором будет содержимое


        self.color = (150, 150, 150)   # цвет фона
        self.textColor = (255, 255, 255)   # цвет текста

        self.cursorColor = (0, 150, 255)   # цвет курсора

        self.oldMousePos = (0, 0)   # старое положение мышки по Y [нужно для скролла]
        self.lastY = 0   # последнее положение текста по Y [нужно для отрисовки текста]

        self.text = ''   # текст text
        self.textPos = 0   # положение текста [нужно для скролла]

        """

        Настройки курсора:

        StartTime - интервал времени, через которое курсор будет появлятся - исчезать
        time - текущее время через которое курсор появится - исчезнет
        direct - указывает исчезнет ли курсор или появится при истекания времени

        """

        self.StartTime = 30
        self.time = 30
        self.direct = -1

        self.eding = True   # указывает на то, что можно ли на данный момент вводить текст в text

        self.fontSize = 30   # размер шрифта
        self.fontPath = None   # путь до шрифта, если == None, то будет использован стандартный
        self.font = pygame.font.Font(self.fontPath, self.fontSize)   # шрифт

        self.mode = 0   # режим отрисовки text
        self.render = True   # переменная, отвечающая за отрисовку text на экране, если == True, то text будет рисоваться, иначе нет.

        self.rect = pygame.Rect(self.x, self.y, self.width, self.height)

        for arg in args:

            if arg == 'x':
                self.x = args[arg]
            elif arg == 'y':
                self.y = args[arg]
                
            elif arg == 'width':
                self.width = args[arg]
            elif arg == 'height':
                self.height = args[arg]

            elif arg == 'color':
                self.color = args[arg]

            elif arg == 'fontSize':
                self.fontSize = args[arg]
            elif arg == 'textColor':
                self.textColor = args[arg]
            
            elif arg == 'font':
                self.fontPath = args[arg]
            
            elif arg == 'cursorColor':
                self.cursorColor = args[arg]

            else:
                print(f'GameUI_Error: "{arg}" is not defined')

        self.font = pygame.font.Font(self.fontPath, self.fontSize)
        self.renderSurface = pygame.Surface((self.width, self.height))
        self.rect = pygame.Rect(self.x, self.y, self.width, self.height)
    
    # функция устанавливающая отрисовку кнопки на экране - (True or False)
    def setRender(self, value):
        if value:
            self.render = True
        elif not value:
            self.render = False
    
    # функция которая добавляет символ в текст
    def Press(self, key):

        """

        Пояснение: переменная [mode] отвечает за режим отрисовки кнопки;

        Если [mode] == 1, то в input может добавляется текст;

        """

        if self.render and self.mode == 1 and self.eding:
            keys = pygame.key.get_pressed()

            # добавление символа в текст
            if not keys[pygame.K_BACKSPACE] and not keys[pygame.K_KP_ENTER] and key != '\r' and self.mode == 1:
                self.text += key
            
            # добавление новой строки в текст
            elif not keys[pygame.K_BACKSPACE] and keys[pygame.K_KP_ENTER] or key == '\r' and self.mode == 1:
                self.text += '\n'

                if self.lastY + self.textPos + self.fontSize // 1.5 > self.rect.height:
                    self.textPos -= self.fontSize * 2
            
            # удаление последнего символа в тексе
            elif keys[pygame.K_BACKSPACE] and self.mode == 1 and self.text != '':
                self.text = self.text[:-1]
                if self.text != '' and self.text[-1] == '\n': self.text = self.text[:-1]
    
    # функция которая сохраняет положение мышки по Y, в дальнейшем из этого получиться
    def Scroll(self):
        self.oldMousePos = pygame.mouse.get_pos()
    
    # функция обвноляющая весь text и взоимодействие с ней
    def update(self):

        if self.render:

            if self.mode == 1:

                """

                Если [direct] == -1, то курсор будет показан
                Иначе, курсор будет спрятан

                """

                if self.direct == -1:
                    self.time -= 1
                    if self.time <= 0: self.direct = 1
                else:
                    self.time += 1
                    if self.time >= self.StartTime: self.direct = -1
            
            if self.rect.width != self.width or self.rect.height != self.height: self.rect.width = self.width; self.rect.height = self.height

            mBT = pygame.mouse.get_pressed()
            mx, my = pygame.mouse.get_pos()

            # как только ЛКМ была нажата, и при этом позиция мыши касается text
            if mBT[0] and self.rect.collidepoint(mx, my):
                newMousePos = pygame.mouse.get_pos()    

                # изменение позиции текста [для скролла]
                if self.textPos + (newMousePos[1] - self.oldMousePos[1]) < 0:
                    self.textPos += (newMousePos[1] - self.oldMousePos[1])
            
            # как только ЛКМ была нажата, и при этом позиция мыши касается text - [mode = 1]
            if mBT[0] and self.rect.collidepoint(mx, my):
                self.mode = 1
            
            # как только ЛКМ была нажата, и при этом позиция мыши не касается text - [mode = 0]
            elif mBT[0] and not self.rect.collidepoint(mx, my):
                self.mode = 0

            self.draw()
    
    def draw(self):

        self.renderSurface = pygame.Surface((self.width, self.height), pygame.SRCALPHA)
        self.renderSurface.fill((0, 0, 0, 0))
        pygame.draw.rect(self.renderSurface, self.color, (0, 0, self.rect.width, self.rect.height))

        if self.mode == 1:

            # подстраивание текста
            while 1:
                get = self.text.split('\n')
                last = self.font.render(get[-1], 1, self.textColor)

                rect = pygame.Rect(10 + last.get_width(), 0, last.get_width(), last.get_height())
                if rect.x > self.rect.width:
                    self.text = self.text[0:-1] + '\n' + self.text[-1]

                    if self.lastY + self.textPos > self.rect.height:
                        self.textPos -= self.fontSize * 2

                    break
                break

        get = self.text.split('\n')

        y = 10

        # отрисовка текста
        for obj in get:

            tx = self.font.render(obj, 1, self.textColor)
            self.renderSurface.blit(tx, (10, y + self.textPos))

            y += self.fontSize // 1.1
        
        self.lastY = y
        
        # отрисовка курсора
        if self.mode == 1 and self.direct == -1 and self.eding:
            pygame.draw.rect(self.renderSurface, self.cursorColor, (10 + tx.get_width() + 5, (y - self.fontSize) + self.textPos, self.fontSize // 10, self.fontSize // 1.1))
        
        # отрисовка всего text
        self.surface.blit(self.renderSurface, (self.rect.x, self.rect.y))

Texts = []


import pygame

pygame.init()

class Button:
    def __init__(self, surface, **args):
        
        Buttons.append(self)
        
        self.x, self.y = 0, 0    # начальоне положение кнопки
        self.width, self.height = 100, 100 # размер кнопки - (ширина, высота)
        self.surface = surface     # окно на котором будет рисоваться кнопка
        
        self.color = (150, 150, 150)   # цвет кнопки
        self.pressedColor = (80, 80, 80)   # цвет кнопки когда она нажата
        self.selectedColor = (120, 120, 120)   # цвет кнопки когда курсор наведен на нее

        self.image = None   # изоброжение кнопки, если == None, то будет рисоваться обычная кнопка с вашимы настройками
        self.imageWidth = None   # ширина изоброжения
        self.imageHeight = None   # высота изоброжения

        self.func = None   # функция которая будет вызываться при нажатии кнопки

        self.defaultText = ''   # текст кнопки
        self.textColor = (255, 255, 255)   # цвет текста
        self.text = self.defaultText
        self.pressedText = self.text   # текст кнопки когда кнопка нажата

        self.fontSize = 50   # размер шрифта
        self.fast = False    # говорит о том будет ли вызываться функция после отжатия кнопки или до тех пор пока кнопка нажата
        self.fontPath = None    # путь до шрифта, если == None, то будет использован стандартный
        self.font = pygame.font.Font(self.fontPath, self.fontSize)

        self.mode = 0   # режим отрисовки кнопки
        self.press = False
        self.render = True  # переменная, отвечающая за отрисовку кнопки на экране, если == True, то кнопка будет рисоваться, иначе нет.
        self.borderRadius = -1  # уровень сглаживания углов у кнопки, если == -1, то сглаживание не будет
        self.fillSize = 0   # уровень заливки кнопки, если == 0, то будет заливаться полностью

        self.rect = pygame.Rect(self.x, self.y, self.width, self.height)

        for arg in args:

            if arg == 'x':
                self.x = args[arg]
            elif arg == 'y':
                self.y = args[arg]
                
            elif arg == 'width':
                self.width = args[arg]
            elif arg == 'height':
                self.height = args[arg]

            elif arg == 'color':
                self.color = args[arg]
            elif arg == 'pressedColor':
                self.pressedColor = args[arg]
            elif arg == 'selectedColor':
                self.selectedColor = args[arg]
            elif arg == 'func':
                self.func = args[arg]
            elif arg == 'text':
                self.defaultText = args[arg]
            elif arg == 'fast':
                self.fast = args[arg]
            elif arg == 'pressedText':
                self.pressedText = args[arg]
            elif arg == 'fontSize':
                self.fontSize = args[arg]
            elif arg == 'textColor':
                self.textColor = args[arg]
            
            elif arg == 'font':
                self.fontPath = args[arg]
            
            elif arg == 'borderRadius':
                self.borderRadius = args[arg]
            elif arg == 'fillSize':
                self.fillSize = args[arg]
            
            elif arg == 'image':
                if args[arg] != None:
                    self.image = pygame.image.load(args[arg]).convert()
                    self.imageWidth = self.image.get_width()
                    self.imageHeight = self.image.get_height()
            elif arg == 'imageWidth':
                self.imageWidth = args[arg]
            elif arg == 'imageHeight':
                self.imageHeight = args[arg]
            
            else:
                print(f'GameUI_Error: "{arg}" is not defined')
            
        self.text = self.defaultText
        self.pressedText = self.text
        self.rect = pygame.Rect(self.x, self.y, self.width, self.height)
        self.font = pygame.font.Font(self.fontPath, self.fontSize)
    
    # функция устанавливающая отрисовку кнопки на экране - (True or False)
    def setRender(self, value):
        if value == True:
            self.render = True
        elif value == False:
            self.render = False
    
    # функция обвноляющая всю кнопку и взоимодействие с ней
    def update(self):
        
        if self.render:

            if self.image != None:
                if self.image.get_width() != self.imageWidth or self.image.get_height() != self.imageHeight: self.image = pygame.transform.scale(self.image, (self.imageWidth, self.imageHeight))
            
            if self.image == None: self.rect.width = self.width; self.rect.height = self.height
            else: self.rect.width = self.imageWidth; self.rect.height = self.imageHeight

            mousePos = pygame.mouse.get_pos()
            mBT = pygame.mouse.get_pressed()
            
            # если переменная fast == False
            if not self.fast:
                """

                Пояснение: переменная [mode] отвечает за режим отрисовки кнопки;

                Если [mode] == 0, то кнопка будет отрисоваться в обычном цвете;
                Если [mode] == 1, то кнопка будет отрисоваться в режиме нажатия на кнопку;
                Если [mode] == 2, то кнопка будет отрисоваться в режиме выбранная кнопка;

                """

                # как только ЛКМ была нажата, и позиция мыши касалась кнопки - [mode = 1]
                if mBT[0] and self.rect.collidepoint(mousePos) and not self.press:
                    self.text = self.pressedText
                    self.mode = 1
                    self.press = True

                # как только ЛКМ была отжата, и позиция мыши не касалась кнопки, и при этом до этого ЛКМ была нажата - [mode = 0]
                if not mBT[0] and self.press and self.rect.collidepoint(mousePos):
                    self.text = self.defaultText
                    if self.func != None:
                        self.func()
                    self.mode = 0
                    self.press = False
                
                # как только позиция мыши не касается кнопки, и при этом ЛКМ была нажата - [mode = 0]
                if not self.rect.collidepoint(mousePos) and self.press and mBT[0]:
                    self.text = self.defaultText
                    self.mode = 0
                    self.press = False
                
                # как только позиция мыши касается кнопки, и при этом ЛКМ не была нажата - [mode = 2]
                if self.rect.collidepoint(mousePos) and not self.press and not mBT[0]:
                    self.mode = 2
                
                # как только позиция мыши не касается кнопки, и ЛКМ не была нажата - [mode = 0]
                if not self.rect.collidepoint(mousePos) and not self.press and not mBT[0]:
                    self.mode = 0
            
            # иначе
            if self.fast:

                # как только ЛКМ была нажата, и позиция мыши касалась кнопки - [mode = 1]
                if mBT[0] and self.rect.collidepoint(mousePos):
                    self.text = self.pressedText
                    self.mode = 1
                    if self.func != None:
                        self.func()
                    self.press = True
                
                # как только ЛКМ была отжата, и при этом позиция мыши касалась кнопки - [mode = 0]
                if not mBT[0] and self.press and self.rect.collidepoint(mousePos):
                    self.text = self.defaultText
                    self.mode = 0
                    self.press = False
                
                # как только позиция мыши не касается кнопки, и при этом ЛКМ была нажата - [mode = 0]
                if not self.rect.collidepoint(mousePos) and self.press and mBT[0]:
                    self.text = self.defaultText
                    self.mode = 0
                    self.press = False
                
                # как только позиция мыши касается кнопки, и при этом ЛКМ не была нажата - [mode = 2]
                if self.rect.collidepoint(mousePos) and not self.press and not mBT[0]:
                    self.mode = 2
                
                # как только позиция мыши не касается кнопки, и при этом ЛКМ не была нажата - [mode = 0]
                if not self.rect.collidepoint(mousePos) and not self.press and not mBT[0]:
                    self.mode = 0

            self.draw()
            
    def draw(self):

        if self.image == None:
            # отрисовка кнопки от зависимости переменной [mode]
            if self.mode == 0:
                pygame.draw.rect(self.surface, self.color, self.rect, self.fillSize, self.borderRadius)
            elif self.mode == 1:
                pygame.draw.rect(self.surface, self.pressedColor, self.rect, self.fillSize, self.borderRadius)
            elif self.mode == 2:
                pygame.draw.rect(self.surface, self.selectedColor, self.rect, self.fillSize, self.borderRadius)
            
            # отрисовка текста кнопки
            if self.text != '':
                text = self.font.render(self.text, 1, self.textColor)
                textWin = text.get_rect(center=(self.rect.x + self.width // 2, self.rect.y + self.height // 2))
                self.surface.blit(text, textWin)
        else:
            self.surface.blit(self.image, self.rect)

            if self.text != '':
                text = self.font.render(self.text, 1, self.textColor)
                textWin = text.get_rect(center=(self.rect.x + self.imageWidth // 2, self.rect.y + self.imageHeight // 2))
                self.surface.blit(text, textWin)

Buttons = []


import pygame

pygame.init()

class Input:
    def __init__(self, surface,  **args):
        inputs.append(self)

        self.x = 0; self.y = 0   # начальное положение input
        self.width = 200; self.height = 50   # размеры input - (ширина, высота)

        self.noText = 'Text'   # текст который будет отоброжаться когда текст input будет пустой
        self.text = ''   # текст input

        self.textColor = (255, 255, 255)   # цвет текста
        self.color = (150, 150, 150)   # цвет input
        self.pressedColor = (100, 100, 100)   # цвет input когда она будет нажата

        self.cursorColor = (0, 150, 255)   # цвет курсора

        self.fontSize = 30   # размер шрифта
        self.fontPath = None   # путь до шрфита, если == None, будет использоваться стандартый

        """

        Настройки курсора:

        StartTime - интервал времени, через которое курсор будет появлятся - исчезать
        time - текущее время через которое курсор появится - исчезнет
        direct - указывает исчезнет ли курсор или появится при истекания времени

        """

        self.StartTime = 30
        self.time = 30
        self.direct = -1

        self.eding = True   # указывает на то, что можно ли на данный момент вводить текст в input

        self.maxChars = 16   # максимальное количество символов в input
        self.textEnd = 0

        self.font = pygame.font.Font(self.fontPath, self.fontSize)   # шрифт

        self.mode = 0   # режим отрисовки текста
        self.render = True   # переменная, отвечающая за отрисовку input на экране, если == True, то input будет рисоваться, иначе нет.
        self.borderRadius = -1   # уровень сглаживания углов у кнопки, если == -1, то сглаживание не будет
        self.fillSize = 0   # уровень заливки кнопки, если == 0, то будет заливаться полностью

        self.surface = surface   # окно на котором будет рисоваться кнопка

        self.rect = pygame.Rect(self.x, self.y, self.width, self.height)

        for arg in args:

            if arg == 'x':
                self.x = args[arg]
            elif arg == 'y':
                self.y = args[arg]
            
            elif arg == 'width':
                self.width = args[arg]
            elif arg == 'height':
                self.height = args[arg]
            
            elif arg == 'noText':
                self.noText = args[arg]
            
            elif arg == 'textColor':
                self.textColor = args[arg]
            elif arg == 'color':
                self.color = args[arg]
            elif arg == 'pressedColor':
                self.pressedColor = args[arg]
            
            elif arg == 'fontSize':
                self.fontSize = args[arg]
            
            elif arg == 'maxChars':
                self.maxChars = args[arg]
            
            elif arg == 'font':
                self.fontPath = args[arg]
            
            elif arg == 'borderRadius':
                self.borderRadius = args[arg]
            elif arg == 'fillSize':
                self.fillSize = args[arg]
            
            elif arg == 'cursorColor':
                self.cursorColor = args[arg]
            
            else:
                print(f'GameUI_Error: "{arg}" is not defined')
        
        self.rect = pygame.Rect(self.x, self.y, self.width, self.height)
        self.font = pygame.font.Font(self.fontPath, self.fontSize)
    
    # функция устанавливающая отрисовку кнопки на экране - (True or False)
    def setRender(self, value):
        if value:
            self.render = True
        elif not value:
            self.render = False
    
    # функция которая добавляет символ в текст
    def Press(self, key):

        """

        Пояснение: переменная [mode] отвечает за режим отрисовки кнопки;

        Если [mode] == 0, то кнопка будет отрисоваться в обычном цвете;
        Если [mode] == 1, то кнопка будет отрисоваться в режиме нажатия на кнопку, и в input может добавляется текст;

        """

        if self.render and self.mode == 1 and self.eding:
            keys = pygame.key.get_pressed()

            # добавление символа в текст
            if not keys[pygame.K_BACKSPACE] and self.mode == 1 and len(self.text) < self.maxChars:
                self.text += key

                # проверка не выходит ли текст за границы input, если да, то текст будет сдвинут влево на один символ
                try:
                    while 1:
                        tx = self.font.render(self.text[self.textEnd:-1] + self.text[-1], 1, (255, 255, 255))
                        rect = pygame.Rect(tx.get_width(), 0, 0, 0)

                        if rect.right + 20 > self.rect.width:
                            self.textEnd += 1
                            continue
                        else:
                            break
                except:
                    pass
            
            # удаление последнего символа в тексте
            elif keys[pygame.K_BACKSPACE] and self.mode == 1 and self.text != '':
                self.text = self.text[:-1]
                if self.textEnd > 0: self.textEnd -= 1
    
    # функция обвноляющая весь input и взоимодействие с ней
    def update(self):
        if self.render:

            if self.mode == 1:

                """

                Если [direct] == -1, то курсор будет показан
                Иначе, курсор будет спрятан

                """

                if self.direct == -1:
                    self.time -= 1
                    if self.time <= 0: self.direct = 1
                else:
                    self.time += 1
                    if self.time >= self.StartTime: self.direct = -1

            mBT = pygame.mouse.get_pressed()
            mx, my = pygame.mouse.get_pos()

            # как только ЛКМ была нажата, и при этом позиция мыши касается input - [mode = 1]
            if mBT[0] and self.rect.collidepoint(mx, my):
                self.mode = 1
            
            # как только ЛКМ была нажата, и при этом позиция мыши не касается input - [mode = 0]
            elif mBT[0] and not self.rect.collidepoint(mx, my):
                self.mode = 0
            
            self.draw()


    def draw(self):

        # отрисовка кнопки от зависимости переменной [mode]
        if self.mode == 0:
            pygame.draw.rect(self.surface, self.color, self.rect, self.fillSize, self.borderRadius)
        elif self.mode == 1:
            pygame.draw.rect(self.surface, self.pressedColor, self.rect, self.fillSize, self.borderRadius)
        
        # если текст в input пустой
        if self.text == '':
            noText = self.font.render(self.noText, 1, self.textColor)
            textWin = noText.get_rect(center=((self.rect.x + noText.get_width() // 2) + 5, self.rect.centery))

            self.surface.blit(noText, textWin)
        
        # иначе
        else:
            Text = self.font.render(self.text[self.textEnd:-1] + self.text[-1], 1, self.textColor)
            textWin = Text.get_rect(center=((self.rect.x + Text.get_width() // 2) + 5, self.rect.centery))

            self.surface.blit(Text, textWin)

            rect = pygame.Rect(Text.get_width(), 0, 0, 0)

            # отрисовка курсора
            if self.mode == 1 and self.direct == -1 and self.eding and rect.x + 20 < self.rect.width:
                pygame.draw.rect(self.surface, self.cursorColor, (self.rect.x + Text.get_width() + 10, self.rect.centery - self.fontSize // 2, self.fontSize // 8, self.fontSize))

inputs = []


#!/usr/bin/python

from setuptools import setup, Extension

Version = '1.6.2'

long_desk = """
Update 1.6.2:
    now the text widget supports transparency;
    
    the fillSize parameter has been added for the button and input;
    
    the cursorColor parameter has been added for the input and text;

Documentation - https://github.com/AlmazCode/game_ui#game_ui
"""

setup(
    name='game_ui',
    version=Version,

    author='AlmazCode',
    author_email='diamondplay43@gmail.com',

    description='UI for pygame',
    long_description=long_desk,

    license='Apache License, Version 2.0, see LICENSE file',

    packages=['game_ui'],

    classifiers=['License :: OSI Approved :: Apache Software License',
                'Operating System :: OS Independent',
                'Intended Audience :: End Users/Desktop',
                'Intended Audience :: Developers',
                'Programming Language :: Python',
                'Programming Language :: Python :: 3',
                'Programming Language :: Python :: 3.6',
                'Programming Language :: Python :: 3.7',
                'Programming Language :: Python :: 3.8',
                'Programming Language :: Python :: 3.9',
                'Programming Language :: Python :: 3.10',
                'Programming Language :: Python :: Implementation :: PyPy',
                'Programming Language :: Python :: Implementation :: CPython'
                ]
)